@@ -9,7 +9,7 @@
 ## |  contiguity constraint.                                    |
 ## |                                                            |
 ## |  Guillaume Guénard, Université de Montréal, Québec, Canada |
-## |  August 2018 - March 2020                                  |
+## |  August 2018 - August 2021                                 |
 ## |                                                            |
 ## \-----------------------------------------------------------*/
 ##
@@ -71,13 +71,18 @@
 #' plotting window. This functionality is to allow one to plot the result of a
 #' constrained clustering over an existing map. In that case, arguments
 #' \code{xlim}, \code{ylim}, \code{axes}, and all other
-#' \link{graphical parameters} to which the method \link{plot} would responds are
-#' ignored.
+#' \link{graphical parameters} to which the method \link{plot} would responds
+#' are ignored.
 #' 
 #' The default colours are generated by function \link{rainbow}; see
 #' \link{palette} for further details on using colour palettes in R. 
 #' The colour palette can be changed by the user.
-#'
+#' 
+#' When disjoint clusters are present (i.e., when the graph provided to
+#' \code{\link{constr.hclust}} is not entirely connected), the function does not
+#' allow one to plot fewer clusters than the number of disjoint subsets; a
+#' warning message is issued to notify the user.
+#' 
 #' @author Guillaume Guénard \email{guillaume.guenard@umontreal.ca}
 #' and Pierre Legendre \email{pierre.legendre@@umontreal.ca}
 #'
@@ -140,6 +145,15 @@ plot.constr.hclust <- function(x, k, xlim, ylim, xlab, ylab, links=FALSE,
         plot(x, cex=cex, ...)
         return(invisible(NULL))
     }
+    if(any(nna <- is.na(x$height)))
+        if(k < sum(nna) + 1L) {
+            warning("Impossible to plot the cluster for k = ", k, " because ",
+                    "the graph provided involves ", sum(nna) + 1L, " non",
+                    "-connected clusters that cannot be linked at any ",
+                    "dissimilarity level. This cluster is plotted for k = ",
+                    sum(nna) + 1L)
+            k <- sum(nna) + 1L
+        }
     cl <- cutree(x,k)
     coords <- x$coords
     if(invert.axes) coords <- coords[,2L:1L]
